/* Config file routines

   OperStats - Copyright (c) 2000-2002 Darcy Grexton
   Contact: skold@habber.net, skold @ HabberNet, ICQ #6285178

   This program is free but copyrighted software; see the file COPYING
   for details.
 */

#include "../inc/operstats.h"

SRA *sralist = NULL;
ULine *ulinelist = NULL;

/* Some vars */
char s_OperServ[NICKLEN], s_StatServ[NICKLEN], s_GlobalNoticer[NICKLEN];
char tlds[64], snoopchan[32], def_akill_reason[256], joinchan[32];
char mirclink[256], mirclinkimg[256], javalinkimg[256], globalmsg[64];
char htmllogo[256];
Boolean_T operserv_on = FALSE, statserv_on = FALSE, globalnoticer_on = FALSE;
Boolean_T globalonakill_on = FALSE, stamplogons_on = FALSE;
Boolean_T logupdates_on = FALSE, dumpchans_on = FALSE, javaclient_on = FALSE;
Boolean_T dumpstats_on = FALSE, learnakills_on = FALSE, dumptlds_on = FALSE;
Boolean_T forgetakills_on = FALSE, globalonlag_on = FALSE, shorthelp_on = FALSE;
Boolean_T globalonauth_on = FALSE, spamserv_on = FALSE, haveserv_on = FALSE;
Boolean_T ignoreulines_on = FALSE, noisyflood_on = FALSE;
int update_timeout, ping_delay, nonwildreq, def_akill_time, unreal_numeric;
int wait_restart, maxclones, maxclonekills, lagthreshold, securitysetting;
int floodmsgs;
time_t floodtime;

/* Add a new SRA into the sra list. */ 
static SRA *new_sra (const char *nick, const char *pass)
{
    SRA *sra;

    sra = scalloc (sizeof (SRA), 1);

    sra->nick = sstrdup (nick);
    sra->pass = sstrdup (pass);

    sra->next = sralist;

    if (sra->next)
        sra->next->prev = sra;

    sralist = sra;

    return sra;
}

/* Add a new U:Line into the U:Lined servers list. */
static ULine *new_uline (const char *name)
{
    ULine *uline;

    uline = scalloc (sizeof (ULine), 1);

    uline->name = sstrdup (name);

    uline->next = ulinelist;
 
    if (uline->next)
	uline->next->prev = uline;
 
    ulinelist = uline;
 
    return uline;
}

/* Load and parse the conf file */
void load_conf (void)
{
    FILE *f = fopen (CONFIGFILE, "r");
    char *item, *s, dBuf[BUFSIZE];

    log (RPL_LOADING, CONFIGFILE);

    if (!f)
    {
	log (ERR_NO_CONFIG);
	exit (1);
    }

    while (fgets (dBuf, 2047, f))
    {
	item = strtok (dBuf, " ");
	strip (item);

	if (*item == '#' || *item == '\t' || *item == ' ')
	    continue;

	if (!stricmp (item, "Info"))
	{
	    s = strtok (NULL, " ");
	    strip (s);

	    if (s && strchr (s, '.'))
		memcpy (me.name, s, sizeof (me.name));
	    else
		fatal ("Config: Info is not set properly: No server name.");

	    s = strtok (NULL, "");
	    strip (s);

	    if (s)
		memcpy (me.desc, s, sizeof (me.desc));
	    else
	    {
		char tmp[25];

		sprintf (tmp, "OperStats %s", version);
		memcpy (me.desc, tmp, sizeof (me.desc));
	    }

	    continue;
	}

	if (!stricmp (item, "BindTo"))
	{
	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
		memcpy (me.bindhost, s, sizeof (me.bindhost));
	    else
		fatal ("Config: BindTo is not set properly: No host.");

	    s = strtok (NULL, " ");   
	    strip (s);

	    if (s)
		me.bindport = atoi (s);
	    else  
		fatal ("Config: BindTo is not set properly: No port.");

	    continue;
	}

	if (!stricmp (item, "Connect"))
	{
	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
		memcpy (me.uplink, s, sizeof (me.uplink));
	    else
		fatal ("Config: Connect is not set properly: No uplink.");

	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
		memcpy (me.pass, s, sizeof (me.pass));
	    else
		fatal ("Config: Connect is not set properly: No password.");

	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
		memcpy (me.ruplink, s, sizeof (me.ruplink));
	    else
		fatal ("Config: Connect is not set properly: No uplink name.");

	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
		me.port = atoi (s);
	    else
		fatal ("Config: Connect is not set properly: No port.");

	    continue;
	}

	if (!stricmp (item, "OperatorService"))
	{
	    /* In case the client info changes */
	    char nick[NICKLEN], user[USERLEN], host[HOSTLEN];
	    char real[REALLEN], mode[12];

	    operserv_on = TRUE;

	    strscpy (nick, s_OperServ, sizeof (nick));
	    strscpy (user, os.user, sizeof (user));
	    strscpy (host, os.host, sizeof (host));
	    strscpy (real, os.real, sizeof (real));
	    strscpy (mode, os.mode, sizeof (mode));

	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
		memcpy (s_OperServ, s, sizeof (s_OperServ));
	    else
		fatal ("Config: OperatorService is not set properly: No nickname.");

	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
		memcpy (os.user, s, sizeof (os.user));
	    else
		fatal ("Config: OperatorService is not set properly: No username.");

	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
		memcpy (os.host, s, sizeof (os.host));
	    else
		fatal ("Config: OperatorService is not set properly: No hostname.");

	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
	    {
		if (!strchr (s, '+'))
		    fatal ("Config: OperatorService: You need a + for modes!");

		memcpy (os.mode, s, sizeof (os.mode));
	    }
	    else
		fatal ("Config: OperatorService is not set properly: No modes.");

	    s = strtok (NULL, "");
	    strip (s);

	    if (s)
		memcpy (os.real, s, sizeof (os.real));
	    else
		fatal ("Config: OperatorService is not set properly: No realname.");

	    if (!(runflags & RUN_STARTING))
	    {
		/* Check if the client info changed. If so, reintroduce the
		   client with the new info.
		 */
		if (os.isonline && (strcmp (s_OperServ, nick) ||
		    strcmp (os.user, user) || strcmp (os.host, host) ||
		    strcmp (os.mode, mode) || strcmp (os.real, real)))
		{
		    send_cmd (nick, "%s :Client info changed",
			me.token ? "," : "QUIT");
		    intro_user (s_OperServ, os.user, os.host, os.real, os.mode);
		}

		/* If OperServ was previously off, and is now on, introduce
		   the pseudo client if needed.
		 */
		if (!os.isonline && operserv_on == TRUE)
		{
		    intro_user (s_OperServ, os.user, os.host, os.real, os.mode);
		    userstat++;
		    os.isonline = 1;
		}
	    }

	    continue;
	}

	if (!stricmp (item, "StatisticalService"))
	{
	    /* In case the client info changes */
	    char nick[NICKLEN], user[USERLEN], host[HOSTLEN];
	    char real[REALLEN], mode[12];

	    statserv_on = TRUE;

	    strscpy (nick, s_StatServ, sizeof (nick));
	    strscpy (user, ss.user, sizeof (user));
	    strscpy (host, ss.host, sizeof (host));
	    strscpy (real, ss.real, sizeof (real));
	    strscpy (mode, ss.mode, sizeof (mode));

	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
		memcpy (s_StatServ, s, sizeof (s_StatServ));
	    else
		fatal ("Config: StatisticalService is not set properly: No nickname.");

	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
		memcpy (ss.user, s, sizeof (ss.user));
	    else
		fatal ("Config: StatisticalService is not set properly: No username.");

	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
		memcpy (ss.host, s, sizeof (ss.host));
	    else
		fatal ("Config: StatisticalService is not set properly: No hostname.");

	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
	    {
		if (!strchr (s, '+'))
		    fatal ("Config: StatisticalService: You need a + for modes!");

		memcpy (ss.mode, s, sizeof (ss.mode));
	    }
	    else
		fatal ("Config: StatisticalService is not set properly: No modes.");

	    s = strtok (NULL, "");
	    strip (s);

	    if (s)
		memcpy (ss.real, s, sizeof (ss.real));
	    else
		fatal ("Config: StatisticalService is not set properly: No realname.");

	    if (!(runflags & RUN_STARTING))
	    {
		/* Check if the client info changed. If so, reintroduce the  
		   client with the new info.
		 */
		if (ss.isonline && (strcmp (s_StatServ, nick) ||
		    strcmp (ss.user, user) || strcmp (ss.host, host) ||
		    strcmp (ss.mode, mode) || strcmp (ss.real, real)))
		{
		    send_cmd (nick, "%s :Client info changed",
			me.token ? "," : "QUIT");
		    intro_user (s_StatServ, ss.user, ss.host, ss.real, ss.mode);
		}

		/* If StatServ was previously off, and is now on, introduce
		   the pseudo client if needed.
		 */
		if (!ss.isonline && statserv_on == TRUE)
		{
		    intro_user (s_StatServ, ss.user, ss.host, ss.real, ss.mode);
		    userstat++;
		    ss.isonline = 1;
		}
	    }

	    continue;
	}

	if (!stricmp (item, "GlobalNoticer"))
	{
	    /* In case the client info changes */
	    char nick[NICKLEN], user[USERLEN], host[HOSTLEN];
	    char real[REALLEN], mode[12];

	    globalnoticer_on = TRUE;

	    strscpy (nick, s_GlobalNoticer, sizeof (nick));
	    strscpy (user, gn.user, sizeof (user));
	    strscpy (host, gn.host, sizeof (host));
	    strscpy (real, gn.real, sizeof (real));
	    strscpy (mode, gn.mode, sizeof (mode));

	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
		memcpy (s_GlobalNoticer, s, sizeof (s_GlobalNoticer));
	    else
		fatal ("Config: GlobalNoticer is not set properly: No nickname.");

	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
		memcpy (gn.user, s, sizeof (gn.user));
	    else
		fatal ("Config: GlobalNoticer is not set properly: No username.");

	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
		memcpy (gn.host, s, sizeof (gn.host));
	    else
		fatal ("Config: GlobalNoticer is not set properly: No hostname.");

	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
	    {
		if (!strchr (s, '+'))
		    fatal ("Config: GlobalNoticer: You need a + for modes!"); 

		memcpy (gn.mode, s, sizeof (gn.mode));
	    }
	    else
		fatal ("Config: GlobalNoticer is not set properly: No modes.");

	    s = strtok (NULL, "");
	    strip (s);

	    if (s)
		memcpy (gn.real, s, sizeof (gn.real));
	    else
		fatal ("Config: GlobalNoticer is not set properly: No realname.");

	    if (!(runflags & RUN_STARTING))
	    {
		/* Check if the client info changed. If so, reintroduce the
		   client with the new info.
		 */
		if (gn.isonline && (strcmp (s_GlobalNoticer, nick) ||
		    strcmp (gn.user, user) || strcmp (gn.host, host) ||
		    strcmp (gn.mode, mode) || strcmp (gn.real, real)))
		{
		    send_cmd (nick, "%s :Client info changed",
			me.token ? "," : "QUIT");
		    intro_user (s_GlobalNoticer, gn.user, gn.host, gn.real,
			gn.mode);
		}

		/* If the GlobalNoticer was previously off, and is now on,
		   introduce the pseudo client if needed.
		 */
		if (!gn.isonline && globalnoticer_on == TRUE)
		{
		    intro_user (s_GlobalNoticer, gn.user, gn.host, gn.real,
			gn.mode);
		    userstat++;
		    gn.isonline = 1;
		}
	    }

	    continue;
	}

	if (!stricmp (item, "SpamService"))
	{
	    /* In case the client info changes */
	    char user[USERLEN], host[HOSTLEN], real[REALLEN];
	    char guest[NICKLEN];
	    int guestnick;

	    spamserv_on = TRUE;

	    strscpy (user, sp.user, sizeof (user));
	    strscpy (host, sp.host, sizeof (host));
	    strscpy (real, sp.real, sizeof (real));

	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
		memcpy (sp.nickpre, s, sizeof (sp.nickpre));
	    else
		fatal ("Config: SpamService is not set properly: No nickname.");

	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
		memcpy (sp.user, s, sizeof (sp.user));
	    else
		fatal ("Config: SpamService is not set properly: No username.");

	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
		memcpy (sp.host, s, sizeof (sp.host));
	    else
		fatal ("Config: SpamService is not set properly: No hostname.");

	    s = strtok (NULL, "");
	    strip (s);

	    if (s)
		memcpy (sp.real, s, sizeof (sp.real));
	    else
		fatal ("Config: SpamService is not set properly: No realname.");

	    /* Everythings ok.. Now generate a nickname to start with. */
	    guestnick = 1+(int) (99999.0*rand()/(RAND_MAX+10000.0));

	    /* Sometimes rand() gives us a 4 digit number. If we get one,
	       we'll add 10000 to it, bringing it into the 5 digit range.
	     */
	    if (guestnick < 10000)
		guestnick = guestnick + 10000;

	    snprintf (guest, sizeof (guest), "%s%d", sp.nickpre, guestnick);
	    strscpy (sp.nick, guest, sizeof (sp.nick));
	    sp.lastnick = time (NULL);

	    if (!(runflags & RUN_STARTING))
	    {
		/* Check if the client info changed. If so, reintroduce the
		   client with the new info.
		 */
		if (sp.isonline && (strcmp (sp.user, user) ||
		    strcmp (sp.host, host) || strcmp (sp.real, real)))
		{
		    send_cmd (sp.nick, "%s :Client info changed",
			me.token ? "," : "QUIT");
		    intro_user (sp.nick, sp.user, sp.host, sp.real, "+");
		}

		/* If SpamServ was previously off, and is now on,
		   introduce the pseudo client if needed.
		 */
		if (!sp.isonline && spamserv_on == TRUE)
		{
		    intro_user (sp.nick, sp.user, sp.host, sp.real, "+");
		    userstat++;
		    sp.isonline = 1;
		}
	    }

	    continue;
	}

	if (!stricmp (item, "SRA"))
	{
	    char *nick, *pass;

	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
		nick = s;
	    else
		nick = "";

	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
		pass = s;
	    else
		pass = "";

	    if (!pass)
		fatal ("Config: Missing password for SRA %s", nick);
	    else
		new_sra (nick, pass);

	    continue;
	}

	if (!stricmp (item, "ULine"))
	{
	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
		new_uline (s);
	    else
		fatal ("Config: Missing server name for U:Line");
 
	    continue;
	}

	if (!stricmp (item, "Trigger"))
	{
	    char *host;
	    int limit;

	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
		host = s;
	    else
		host = "";

	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
		limit = atoi (s);
	    else
		limit = 0;

	    if (!limit)
		fatal ("Config: Missing limit for Trigger %s", host);
	    else
		new_trigger (host, limit);

	    continue;
	}

	if (!stricmp (item, "Exception"))
	{
	    char *host;

	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
		host = s;
	    else
		host = "";

	    new_exception (host);

	    continue;
	}

	if (!stricmp (item, "SyncTime"))
	{
	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
		update_timeout = dotime (s);  
	    else
		fatal ("Config: SyncTime is not set properly: No time value.");

	    continue;
	}

	if (!stricmp (item, "MaxConnections"))
	{
	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
		maxclones = atoi (s);
	    else
		fatal ("Config: MaxConnections is not set properly: No value.");

	    continue;
	}

        if (!stricmp (item, "MaxCloneKills"))
        {
	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
		maxclonekills = atoi (s);
	    else
		fatal ("Config: MaxCloneKills is not set properly: No value.");

	    continue;
	}

	if (!stricmp (item, "PingDelay"))
	{
	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
		ping_delay = dotime (s);
	    else
		fatal ("Config: PingDelay is not set properly: No time value.");

	    continue;
	}

	if (!stricmp (item, "SnoopChan"))
	{
	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
		memcpy (snoopchan, s, sizeof (snoopchan));
	    else
		fatal ("Config: SnoopChan is not set properly: No channel.");

	    continue;
	}

	if (!stricmp (item, "SecuritySetting"))
	{
	    s = strtok (NULL, "");
	    strip (s);

	    if (s)
	    {
		if (!stricmp (s, "All"))
		    securitysetting = 1;
		if (!stricmp (s, "Passwords"))
		    securitysetting = 2;
		if (!stricmp (s, "None"))
		    securitysetting = 0;
	    }
	    else
		fatal ("Config: SecuritySetting is not set properly: No value.");

	    continue;
	}

	if (!stricmp (item, "GlobalMSG"))
	{
	    s = strtok (NULL, "");
	    strip (s);

	    if (s)
		memcpy (globalmsg, s, sizeof (globalmsg));
	    else
		fatal ("Config: GlobalMSG is not set properly: No message.");

	    continue;
	}

	if (!stricmp (item, "JoinChan"))
	{
	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
		memcpy (joinchan, s, sizeof (joinchan));
	    else
		fatal ("Config: JoinChan is not set properly: No channel.");

	    continue;
	}

	if (!stricmp (item, "mIRCLink"))
	{
	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
		memcpy (mirclink, s, sizeof (mirclink));
	    else
		fatal ("Config: mIRCLinks is not set properly: No address.");

	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
		memcpy (mirclinkimg, s, sizeof (mirclinkimg));
	    else
		fatal ("Config: mIRCLinks is not set properly: No image.");

	    continue;
	}

	if (!stricmp (item, "JavaClient"))
	{
	    javaclient_on = TRUE;

	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
		memcpy (javalinkimg, s, sizeof (javalinkimg));
	    else
		fatal ("Config: JavaClient is not set properly. No Image.");

	    continue;
	}

	if (!stricmp (item, "HTMLLogo"))
	{
	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
		memcpy (htmllogo, s, sizeof (htmllogo));
	    else
		fatal ("Config: HTMLLogo is not set properly. No image.");

	    continue;
	}

	if (!stricmp (item, "TLD"))
	{  
	    s = strtok (NULL, "");
	    strip (s);

	    if (s)
		memcpy (tlds, s, sizeof (tlds));
	    else
		fatal ("Config: TLD is not set properly: No TLDs");

	    continue;
        }

	if (!stricmp (item, "AKillExpire"))
	{
	    s = strtok (NULL, "");
	    strip (s);

	    if (s)
		def_akill_time = dotime (s);
	    else
		fatal ("Config: AKillExpire is not set properly: No time value.");

	    continue;
	}

	if (!stricmp (item, "NonWildCards"))
	{
	    s = strtok (NULL, "");
	    strip (s);

	    if (s)
		nonwildreq = atol (s);
	    else
		fatal ("Config: NonWildCards is not set properly: No value.");

	    continue;
	}

#ifdef UNREAL3
	if (!stricmp (item, "UnrealNumeric"))
	{
	    s = strtok (NULL, "");
	    strip (s);

	    if (s)
		unreal_numeric = atol (s);
	    else
		fatal ("Config: UnrealNumeric is not set properly: No value.");

	    continue;
	}
#endif

	if (!stricmp (item, "DefaultAKillReason"))
	{
	    s = strtok (NULL, "");
	    strip (s);

	    if (s)
		memcpy (def_akill_reason, s, sizeof (def_akill_reason));
	    else
		fatal ("Config: DefaultAKillReason is not set properl: No reason.");

	    continue;
	}

	if (!stricmp (item, "WaitRestart"))
	{
	    s = strtok (NULL, "");
	    strip (s);

	    if (s)
		wait_restart = dotime (s);
	    else
		fatal ("Config: WaitRestart is not set properly: No time value.");

	    continue;
	}

	if (!stricmp (item, "LagThreshold"))
	{
	    s = strtok (NULL, "");
	    strip (s);

	    if (s)
		lagthreshold = dotime (s);
	    else
		fatal ("Config: LagThreshold is not set properly: No time value.");

	    continue;
	}

	if (!stricmp (item, "FloodTrigger"))
	{
	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
		floodmsgs = atoi (s);
	    else
		fatal ("Config: FloodTrigger is not set properly: No message amount.");

	    s = strtok (NULL, " ");
	    strip (s);

	    if (s)
		floodtime = dotime (s);
	    else
		fatal ("Config: FloodTrigger is not set properly: No message time.");

	    continue;
	}

        if (!stricmp (item, "GlobalOnAKill"))
	{
	    globalonakill_on = TRUE;
	    continue;
	}

	if (!stricmp (item, "GlobalOnLag"))
	{
	    globalonlag_on = TRUE;
	    continue;
	}

	if (!stricmp (item, "GlobalOnAuth"))
	{
	    globalonauth_on = TRUE;
	    continue;
	}

	if (!stricmp (item, "LearnAKills"))
	{
	    learnakills_on = TRUE;
	    continue;
	}

	if (!stricmp (item, "ForgetAKills"))
	{
	    forgetakills_on = TRUE;
	    continue;
	}

        if (!stricmp (item, "StampLogons"))
	{
	    stamplogons_on = TRUE;
	    continue;
	}

	if (!stricmp (item, "HaveServ"))
	{
	    haveserv_on = TRUE;
	    continue;
	}

	if (!stricmp (item, "ShortHelp"))
	{
	    shorthelp_on = TRUE;
	    continue;
	}

	if (!stricmp (item, "LogUpdates"))
	{
	    logupdates_on = TRUE;
	    continue;
	}

	if (!stricmp (item, "DumpChans"))
	{
	    dumpchans_on = TRUE;
	    continue;
	}

	if (!stricmp (item, "DumpStats"))
	{
	    dumpstats_on = TRUE;
	    continue;
	}

	if (!stricmp (item, "DumpTLDs"))
	{
	    dumptlds_on = TRUE;
	    continue;
	}

	if (!stricmp (item, "IgnoreULines"))
	{
	    ignoreulines_on = TRUE;
	    continue;
	}

	if (!stricmp (item, "NoisyFlood"))
	{
	    noisyflood_on = TRUE;
	    continue;
	}
    }

    fclose (f);
}

/* Sanity checks on the config options */
int check_conf (void)
{
    SRA *sra;
    int i = 0;

    for (sra = sralist; sra; sra = sra->next)
	i++;

    if (!i)
	fatal ("Config: You must have at least one SRA.");

    if (operserv_on == FALSE && statserv_on == FALSE)
	fatal ("Config: OperatorService and StatisticalService cannot both be disabled.");

    if (operserv_on == FALSE)
    {
	globalnoticer_on = FALSE;
	maxclones = 0;
	maxclonekills = 0;
	learnakills_on = FALSE;
	forgetakills_on = FALSE;
	globalonakill_on = FALSE;
	stamplogons_on = FALSE;
    }

    if (statserv_on == FALSE)
    {
	dumpchans_on = FALSE;
	dumpstats_on = FALSE;
	dumptlds_on = FALSE;
	javaclient_on = FALSE;
    }

    if (!update_timeout)
	update_timeout = 1200;

    if (!ping_delay)
	ping_delay = 60;

    if (!lagthreshold || lagthreshold < 1)
	lagthreshold = 20;

    if (!def_akill_time)
	def_akill_time = 604800; /* One week */

    if (!nonwildreq)
	nonwildreq = 5;

    if (update_timeout < 60)
	printf ("Warning: Setting SyncTime less than 1 minute is unwise!\n");

    if (wait_restart < 5)
	fatal ("Config: WaitRestart cannot be less than 5s.");

    if (!strlen (tlds) && (globalnoticer_on == TRUE))
    {
        printf ("Config: TLD is not set properly. GlobalNoticer disabled.\n");
	globalnoticer_on = FALSE;
    }

    if (!strlen (def_akill_reason))
	memcpy (def_akill_reason, "You are banned.", sizeof (def_akill_reason));

#ifdef UNREAL3
    if (!unreal_numeric)
	fatal ("Config: UnrealNumeric is required for UnrealIRCd.");

    if (unreal_numeric < 1 || unreal_numeric > 255)
	fatal ("Config: UnrealNumeric is not set properly: Must be between 1 and 255.");
#endif

    return 1;
}
